
#define AT91_PTRS	1
#include "../at91m40400.h"

#define TRUE	1
#define FALSE	0
#define RED_LED		(1<<1)
#define GREEN_LED	(1<<2)
#define AMBER_LED	(1<<4)

extern unsigned short prog_array[];
extern long prog_array_size();
/* Function prototypes */

void flash_ok();
void flash_not_ok();
void leds_on(void);
void leds_off(void);
void chip_erase();
void get_flash_id();
void sector_erase(short sector);
void block_erase(short block);
void program_word(long address,unsigned short word);
unsigned short read_word(long address);

int main (void)
{
	int i;
	int ok;


	at91_pio->PIO_OER = 0x2 | 0x4 | 0x10;
	leds_on();

	get_flash_id();
	chip_erase();

	for(i=0;i<prog_array_size();i++)
		program_word((long)i,prog_array[i]);
	leds_off();
	ok = TRUE;
	for(i=0;i<prog_array_size();i++)
		if (prog_array[i] != read_word((long)i))
			ok = FALSE;

	if (ok == TRUE)
		flash_ok();
	else
		flash_not_ok();
	return(0);
}

void flash_ok()
{	
	int n,x;

	while(1)
	{
		leds_on();
		for(n=0;n<0xFFFF;n++)
			x++;
		for(n=0;n<0xFFFF;n++)
			x++;
		leds_off();
		for(n=0;n<0xFFFF;n++)
			x++;
	}
}

void flash_not_ok()
{	
	int n,x;

	while(1){
		leds_on();
		for(n=0;n<0xFFFF;n++)
			x++;
		leds_off();
		for(n=0;n<0xFFFF;n++)
			x++;
		for(n=0;n<0xFFFF;n++)
			x++;
	}
}

void leds_on(void)
{
	at91_pio->PIO_SODR = (RED_LED | GREEN_LED | AMBER_LED);
}

void leds_off(void)
{
	at91_pio->PIO_CODR = (RED_LED | GREEN_LED | AMBER_LED);
}

void __gccmain()
{
}

void get_flash_id()
{
	unsigned short *ptr,id,man;
	
	ptr = (unsigned short *)FLASH_BASE;

	ptr[0x5555] = 0xaa;
	ptr[0x2aaa] = 0x55;
	ptr[0x5555] = 0x90;

	id = ptr[1];
	man = ptr[0];

	ptr[0x5555] = 0xaa;
	ptr[0x2aaa] = 0x55;
	ptr[0x5555] = 0xf0;
}

void chip_erase()
{
	unsigned short *ptr;
	int i;
	
	ptr = (unsigned short *)FLASH_BASE;

	ptr[0x5555] = 0xaa;
	ptr[0x2aaa] = 0x55;
	ptr[0x5555] = 0x80;
	ptr[0x5555] = 0xaa;
	ptr[0x2aaa] = 0x55;
	ptr[0x5555] = 0x10;
	i = 0;
	while(i != 2)
		if (ptr[0] != 0xffff)
			i = 0;
		else
			i++;
}

void sector_erase(short sector)
{
	unsigned short *ptr;
	
	ptr = (unsigned short *)FLASH_BASE;

	ptr[0x5555] = 0xaa;
	ptr[0x2aaa] = 0x55;
	ptr[0x5555] = 0x80;
	ptr[0x5555] = 0xaa;
	ptr[0x2aaa] = 0x55;
	ptr[sector] = 0x30;
}

void block_erase(short block)
{
	unsigned short *ptr;
	
	ptr = (unsigned short *)FLASH_BASE;

	ptr[0x5555] = 0xaa;
	ptr[0x2aaa] = 0x55;
	ptr[0x5555] = 0x80;
	ptr[0x5555] = 0xaa;
	ptr[0x2aaa] = 0x55;
	ptr[block] = 0x50;
}

void program_word(long address,unsigned short word)
{
	unsigned short *ptr;
	int i;
	
	ptr = (unsigned short *)FLASH_BASE;

	ptr[0x5555] = 0xaa;
	ptr[0x2aaa] = 0x55;
	ptr[0x5555] = 0xa0;
	ptr[address] = word;

	i = 0;
	while(i != 2)
		if (ptr[address] != word)
			i = 0;
		else
			i++;
	
}

unsigned short read_word(long address)
{
	unsigned short *ptr;
	
	ptr = (unsigned short *)FLASH_BASE;

	return(ptr[address]);
}

